"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pluralize = require("pluralize");
const utility_1 = require("./utility");
function generateFilter(type, options, filterType, domainType) {
    'use strict';
    const tsDomainType = options.tsTypeMap[domainType] ? options.tsTypeMap[domainType] : domainType;
    const tsConstructorParameters = [];
    const filterParameters = [];
    if (type.constructors.length === 1) {
        for (const parameter of type.constructors[0].parameters) {
            let tsParameterType = utility_1.Utility.translateType(parameter.type.name, options);
            if (tsParameterType) {
                if (parameter.name === 'fieldName') {
                    tsConstructorParameters.push(`private ${parameter.name}: (i: ${domainType}) => any`);
                    filterParameters.push(`getPropertyName(this.${parameter.name})`);
                }
                else {
                    const shouldEncode = tsParameterType === 'string' || tsParameterType === 'any' || tsParameterType === 'Date';
                    const shouldToString = tsParameterType !== 'string';
                    const toString = tsParameterType === 'Date' ? 'toISOString' : 'toString';
                    if (parameter.type.isCollection) {
                        tsParameterType += '[]';
                    }
                    const tsConstructorParameter = parameter.defaultValue ?
                        `private ${parameter.name}: ${tsParameterType} = ${parameter.defaultValue}` :
                        `private ${parameter.name}: ${tsParameterType}`;
                    tsConstructorParameters.push(tsConstructorParameter);
                    let filterParameter;
                    if (parameter.type.isCollection) {
                        let mapToExpression = 'i';
                        if (shouldToString) {
                            mapToExpression = `i.${toString}()`;
                        }
                        if (shouldEncode) {
                            mapToExpression = `encodeURIComponent(${mapToExpression})`;
                        }
                        const mapCall = mapToExpression !== 'i' ? `.map(i => ${mapToExpression})` : '';
                        filterParameter = `this.${parameter.name}${mapCall}.join(',')`;
                    }
                    else {
                        filterParameter = `this.${parameter.name}`;
                        if (shouldToString) {
                            filterParameter = `${filterParameter}.${toString}()`;
                        }
                        if (shouldEncode) {
                            filterParameter = `encodeURIComponent(${filterParameter})`;
                        }
                    }
                    if (parameter.defaultValue === 'null') {
                        filterParameter = `this.${parameter.name} ? ${filterParameter} : null`;
                    }
                    filterParameters.push(filterParameter);
                }
            }
        }
    }
    return `
export class ${pluralize(domainType)}${type.name}${filterType} extends ${filterType}<${tsDomainType}> {
  constructor(${tsConstructorParameters.join(', ')}) {
    super();
  }

  public getFilterName(): string {
    return '${type.name}';
  }

  public getParameters(): string[] {
    return [${filterParameters.join(', ')}];
  }

  protected __dummy(): ${tsDomainType} {
    return null;
  }
}`.trim();
}
exports.generateFilter = generateFilter;
//# sourceMappingURL=generate-filter.js.map