"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utility_1 = require("./utility");
function generateInterface(type, options) {
    'use strict';
    let baseClass;
    if (type.inherits) {
        const baseClasses = type.inherits
            .filter(i => i && i.length && i[0] !== 'I');
        baseClass = baseClasses.length === 1 ? baseClasses[0] : '';
    }
    const tsExtends = baseClass ? ` extends ${baseClass}` : '';
    const propertyStrings = [];
    for (const property of type.properties) {
        let tsPropertyName = utility_1.Utility.transfromPropertyName(property.name);
        if (property.type.isNullable || (options.allPropertiesOptional && tsPropertyName !== 'id')) {
            tsPropertyName += '?';
        }
        let tsType = utility_1.Utility.translateType(property.type.name, options) || property.type.name;
        if (property.type.isCollection) {
            tsType += '[]';
        }
        propertyStrings.push(`${tsPropertyName}: ${tsType}`);
    }
    let properties = '';
    if (type.properties.length > 0) {
        properties = `  ${propertyStrings.join(';\n  ')};\n`;
    }
    return `export interface ${type.name}${tsExtends} {\n${properties}}`;
}
exports.generateInterface = generateInterface;
//# sourceMappingURL=generate-interface.js.map