"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pluralize = require("pluralize");
const c_sharp_objects_1 = require("./c-sharp-objects");
const c_sharp_parser_1 = require("./c-sharp-parser");
const utility_1 = require("./utility");
function generateSchema(inputs) {
    let result = '';
    const types = [].concat.apply([], inputs.map(input => c_sharp_parser_1.CSharpParser.parse(input)))
        .filter(type => type instanceof c_sharp_objects_1.CSharpEnum === false)
        .map(type => type)
        .filter(type => isEntityType(type));
    const typeNames = types.map(type => type.name);
    result += 'export function getSchema(schema: any) {\n';
    result += '  const schemas = {\n';
    const schemas = types
        .map(type => pluralize(utility_1.Utility.transfromPropertyName(type.name)))
        .map(name => `    ${name}: new schema.Entity('${name}')`);
    result += `${schemas.join(',\n')}\n`;
    result += '  }\n\n';
    for (const type of types) {
        const name = pluralize(utility_1.Utility.transfromPropertyName(type.name));
        const relations = type.properties.filter(property => typeNames.indexOf(property.type.name) >= 0);
        if (relations.length) {
            result += `  schemas['${name}'].define({\n`;
            const definition = relations
                .map(relation => {
                const propertyName = utility_1.Utility.transfromPropertyName(relation.name);
                const typeName = pluralize(utility_1.Utility.transfromPropertyName(relation.type.name));
                return relation.type.isCollection ?
                    `    ${propertyName}: [ schemas['${typeName}'] ]` : `    ${propertyName}: schemas['${typeName}']`;
            });
            result += `${definition.join(',\n')}\n`;
            result += '  });\n\n';
        }
    }
    result += '  return schemas;\n';
    result += '}\n';
    return result;
}
exports.generateSchema = generateSchema;
function isEntityType(type) {
    return type.properties.find(property => property.name === 'Id')
        || type.name.endsWith('Detail')
        || type.namespace.includes('Dto');
}
//# sourceMappingURL=generate-schema.js.map