"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const c_sharp_objects_1 = require("./c-sharp-objects");
const c_sharp_parser_1 = require("./c-sharp-parser");
const options_1 = require("./options");
const generate_enum_1 = require("./generate-enum");
const generate_filter_1 = require("./generate-filter");
const generate_interface_1 = require("./generate-interface");
const dtoFilterRegex = /DtoFilter<(?:Dmn\.)?([\w]+)/;
const primaryDtoFilterRegex = /PrimaryDtoFilter<(?:Dmn\.)?([\w]+)/;
const elasticDtoFilterRegex = /ElasticDtoFilter<(?:Dmn\.)?([\w]+)/;
const bypassElasticDtoFilterRegex = /BypassElasticDtoFilter<(?:Dmn\.)?([\w]+)/;
function tsGenerator(input, options = null) {
    const results = [];
    options = Object.assign({}, options_1.defaultOptions, options);
    const types = c_sharp_parser_1.CSharpParser.parse(input);
    for (const type of types) {
        const dtoFilterMatch = type.inherits
            .filter(inherit => inherit.match(primaryDtoFilterRegex) === null)
            .filter(inherit => inherit.match(elasticDtoFilterRegex) === null)
            .filter(inherit => inherit.match(bypassElasticDtoFilterRegex) === null)
            .map(inherit => inherit.match(dtoFilterRegex))
            .find(match => match !== null);
        const primaryDtoFilterMatch = type.inherits
            .map(inherit => inherit.match(primaryDtoFilterRegex))
            .find(match => match !== null);
        const elasticDtoFilterMatch = type.inherits
            .filter(inherit => inherit.match(bypassElasticDtoFilterRegex) === null)
            .map(inherit => inherit.match(elasticDtoFilterRegex))
            .find(match => match !== null);
        const bypassElasticDtoFilterMatch = type.inherits
            .map(inherit => inherit.match(bypassElasticDtoFilterRegex))
            .find(match => match !== null);
        const isFilter = type.inherits.some(inherit => inherit.includes('Filter'));
        if (type instanceof c_sharp_objects_1.CSharpEnum) {
            results.push(generate_enum_1.generateEnum(type));
        }
        else if (dtoFilterMatch && !primaryDtoFilterMatch) {
            results.push(generate_filter_1.generateFilter(type, options, 'Filter', dtoFilterMatch[1]));
        }
        else if (dtoFilterMatch && primaryDtoFilterMatch) {
            results.push(generate_filter_1.generateFilter(type, options, 'Filter', dtoFilterMatch[1]));
            results.push(generate_filter_1.generateFilter(type, options, 'PrimaryFilter', dtoFilterMatch[1]));
        }
        else if (primaryDtoFilterMatch) {
            results.push(generate_filter_1.generateFilter(type, options, 'PrimaryFilter', primaryDtoFilterMatch[1]));
        }
        else if (elasticDtoFilterMatch) {
            results.push(generate_filter_1.generateFilter(type, options, 'ElasticFilter', elasticDtoFilterMatch[1]));
        }
        else if (bypassElasticDtoFilterMatch) {
            results.push(generate_filter_1.generateFilter(type, options, 'BypassElasticFilter', bypassElasticDtoFilterMatch[1]));
        }
        else if (isFilter === false) {
            results.push(generate_interface_1.generateInterface(type, options));
        }
    }
    return results.join('\n\n');
}
exports.tsGenerator = tsGenerator;
//# sourceMappingURL=tsgen.js.map